# Copyright 2015 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.  You may obtain a copy
# of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os
import sys

# Program name to use for log messages.
PROGRAM_NAME = os.path.basename(sys.argv[0])


def _build_logger():
    """Instantiates a global logger for the program.

    Returns:
        Logger.  The logger instance to use for the application.
    """
    handler = logging.StreamHandler()

    formatter = logging.Formatter(PROGRAM_NAME + ': %(levelname)s: %(message)s')
    handler.setFormatter(formatter)

    logger = logging.getLogger(PROGRAM_NAME)
    logger.addHandler(handler)
    logger.setLevel(logging.WARNING)
    return logger


# Global logger instance for the application.
LOGGER = _build_logger()
